package IPTC::ExifUtilsWriter;

use version; our $VERSION = qv('0.4_3');
use strict;
use Readonly;

use Image::File;
use Util::Logfile;
use Util::System;

require 'globals.pl';

# Endungen der Dateien, die exifedit bearbeiten kann
# kleingeschrieben
Readonly my %file_suffixes_writeable => (
    'jpg' => 1,
    'tif' => 1,
    'cr2' => 1,
    'dcr' => 1,
    'nef' => 1,
);

sub new {
    my ( $class, $option_praefix ) = @_;
    my $self = {};

    bless $self, $class;
    $self->{option_praefix} = $option_praefix;
    $main::log = Util::Logfile->new() if not defined $main::log;

    return $self;
}

sub can_write {
    my ( $self, $filename ) = @_;
    my $suffix = $filename;

    $suffix =~ s/.*\.//;
    $suffix = lc $suffix;

    return $file_suffixes_writeable{$suffix};
}

sub write_file {
    my ( $self, $filename, $iptc_data_hash_ref ) = @_;

    $main::log->fail("Benoetige IPTC-Daten!")
        if not defined $iptc_data_hash_ref;
    $main::log->fail("Dateiname fehlt!") if not $filename;
    $main::log->fail("Format von '$filename' kann nicht geschrieben werden!")
        if not $self->can_write($filename);
    $main::log->fail("Datei '$filename' existiert nicht!") if not -f $filename;

    $self->{iptc_data_hash_ref} = $iptc_data_hash_ref;

    my $command = $self->_get_command($filename);
    $main::log->log_verbose(
        "IPTC::ExifUtilsWriter: Fhre Befehl aus: '$command'");

    return execute_check_error($command)
        && $self->_set_date_time_taken($filename)
        && Image::File::set_equal_time_to_xmp_file($filename);
}

sub _set_date_time_taken {
    my ( $self, $filename ) = @_;
    my $modify_date_command = $self->_get_modify_date_command($filename);

    return 1 if not $main::SET_IMAGEFILE_MODIFY_TIME_FROM_SHOOTING_TIME;

    $main::log->log_verbose(
        "Setze Aufnahmezeit als Dateimodifikationsdatum bei '$filename");
    return execute_check_error($modify_date_command);
}

sub execute_check_error {
    my ($command) = @_;
    my @command_output = ();

    if (!Util::System::execute_check_error(
            $command, 'error', \@command_output
        )
        )
    {
        foreach my $line (@command_output) {
            $main::log->log_print( $line, 1 );
        }
        return 0;
    }
    return 1;
}

# Liefert den Befehl 'exifedit ...', der ber das Betriebssystem
# ausgefhrt wird
#
sub _get_command {
    my ( $self, $filename ) = @_;
    my $command = "exifedit"                    # Programmname
        . " $self->{option_praefix}q"           # Quiet
        . $self->_get_backup_option()           # Kein Backup
        . $self->_get_remove_fields_option()    # Alle Felder lschen
        . " $self->{option_praefix}a \"";       # Hinzufgen folgender Felder

    foreach my $iptc_key ( keys %main::IPTC_INFO ) {
        my $option = $main::IPTC_INFO{$iptc_key}->{ExifUtils};
        next if not defined $option;
        my $value = $self->{iptc_data_hash_ref}->{$iptc_key};
        if ( main::is_iptc_value_repeatable($iptc_key) ) {
            _add_array_to_options( \$command, $option, $value );
        }
        else {
            _add_scalar_to_options( \$command, $option, $value );
        }
    }

    $command .= "\" $filename";

    return $command;
}

sub _get_backup_option {
    my ($self) = @_;

    return " $self->{option_praefix}b";
}

sub _get_remove_fields_option {
    my ($self)       = @_;
    my $command      = " $self->{option_praefix}r ";
    my $option_count = 0;

    foreach my $iptc_key ( keys %main::IPTC_INFO ) {
        my $option = $main::IPTC_INFO{$iptc_key}->{ExifUtils};
        next if not defined $option;
        $command .= $option_count++ > 0 ? ",$option" : "\"$option";
    }

    return $command . '"';
}

sub _add_array_to_options {
    my ( $cmd_ref, $option, $array_ref, $is_first ) = @_;
    my $index = 0;

    ${$cmd_ref} .= ", $option=\\\"";
    foreach my $element ( @{$array_ref} ) {
        if ( $index++ > 0 ) {
            ${$cmd_ref} .= ';';
        }
        _escape( \$element, 1 );
        ${$cmd_ref} .= $element;
    }
    ${$cmd_ref} .= '\"';
}

sub _add_scalar_to_options {
    my ( $cmd_ref, $option, $value, $is_first ) = @_;

    if ($value) {
        _escape( \$value );
        ${$cmd_ref} .= ", $option=\\\"$value\\\"";
    }
}

# Maskiert Kommandozeilen-Separatoren
# Ersetzt " -> ' und bei sich wiederholenden Elementen ; durch Leerzeichen
#
sub _escape {
    my ( $string_ref, $is_array ) = @_;

    ${$string_ref} =~ s/\"/'/g;

    if ($is_array) {
        ${$string_ref} =~ s/\;/ /g;
    }
}

sub _get_modify_date_command {
    my ( $self, $filename ) = @_;

    return "exiffile $self->{option_praefix}t $filename";
}

1;

__END__

=head1 NAME

IPTC::ExifUtilsWriter - Schreiben von IPTC-Daten mit den C<EXIFutils>

=head1 DESCRIPTION

Die Klasse C<IPTC::ExifUtilsWriter> schreibt IPTC-Daten mit den C<EXIFutils>
(L<http://www.hugsan.com/EXIFutils/>). I<Die Programme C<exifedit> und
C<exiffile> der EXIFutils mssen im Pfad fr ausfhrbare Dateien stehen!>

=head1 METHODS

=over

=item $iptc_writer = new IPTC::ExifUtilsWriter($option_praefix_string)

Erzeugt ein neues Objekt

Der 1. Parameter ist ein String mit dem Optionsprfix - einem Bindestrich
unter Linux und einem Schrgstrich unter Windows

=item $can_write = $iptc_writer->can_write($image_filename)

Liefert, ob IPTC-Daten in eine Datei geschrieben werden knnen; d.h.,
ob die C<exifutils> in eine Datei dieses Formats IPTC-Daten schreiben knnen.

Das Dateiformat wird der Einfachheit halber anhand der Endung erkannt.

Der 1. Parameter ist ein String mit dem Namen der Bilddatei

Die Rckgabe ist true, wenn C<exifedit> IPTC-Daten in die Datei schreiben
kann

=item $iptc_writer->write_file($filename, $iptc_data_hash_ref)

Schreibt IPTC-Daten in eine Datei.

Der 1. Parameter ist ein String mit dem Dateinamen

Der 2. Parameter ist ein Hash mit den zu schreibenden IPTC-Daten

Die Rckgabe ist true bei erfolgreichem Schreiben

=back

=head1 BUGS

=over

=item Rckgabewert von exifedit

C<exifedit> liefert (manchmal/immer?) auch bei Fehlschlgen true,
infolgedessen auch C<write_file()>. 

=item exifedit und escapte Zeichen

C<exifedit> meldet Fehler bei richtig escapten Anfhrungszeichen

Escapte Kommas fhren zu Fehlern, deshalb nicht escaped

Escapte Gleichheitszeichen nicht getestet, weggelassen

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/05/18

=cut
