package Util::Logfile;

use strict;
use File::Copy;
use version; our $VERSION = qv('0.1_6');

use Util::String;

sub new {
    my ( $class, $filename, $append ) = @_;
    my $self = {};

    bless $self, $class;

    $self->{logfile_max_size} = 1048576;    # 1 MB
    $self->{verbose} = 0;

    if ( defined $filename ) {
        $self->open( $filename, $append );
    }

    return $self;
}

sub DESTROY {
    my ($self) = @_;

    if ( $self->{fh} ) {
        close $self->{fh};
    }
}

sub set_verbose {
    my ($self, $verbose) = @_;
    
    $self->{verbose} = $verbose;
}

sub set_logfile_max_size {
    my ( $self, $logfile_max_size ) = @_;

    $self->{logfile_max_size} = $logfile_max_size;
}

sub open {
    my ( $self, $filename, $append ) = @_;

    $self->{filename} = $filename;

    my $logfile_backuped = $self->_backup_logfile();

    if ( $self->{fh} ) {
        my $old_fh = $self->{fh};
        close $old_fh;
        undef $self->{fh};
    }

    my $fh;
    my $open_type = defined $append && $append ? '>>' : '>';
    open $fh, $open_type, $filename;

    if ($fh) {
        $self->{fh} = $fh;
        print $fh "\n=== Starte Logging, " . _get_time() . " ===\n\n";
        if ($logfile_backuped) {
            $self->_print_log_logfile_backuped();
        }
        return 1;
    }

    return 0;
}

sub open_append {
    my ( $self, $filename ) = @_;

    return $self->open( $filename, 1 );
}

sub log_print {
    my ( $self, $message, $error ) = @_;

    Util::String::trim( \$message );
    if ($error) {
        print STDERR "$message\n";
    }
    else {
        print STDOUT "$message\n";
    }

    $self->log( $message, $error );
}

sub log {
    my ( $self, $message, $error ) = @_;

    return if not defined $self->{fh};

    Util::String::trim( \$message );
    $message = _get_time() . ( $error ? ' [Error] ' : ' ' ) . $message . "\n";
    my $fh = $self->{fh};

    print $fh $message;
}

sub log_verbose {
    my ($self, $message) = @_;
    
    if ($self->{verbose}) {
        print "$message\n";
    }
}

sub fail {
    my ( $self, $message ) = @_;

    $self->log_print( $message, 1 );

    if ( $self->{fh} ) {
        close $self->{fh};
        undef $self->{fh};
    }

    exit 1;
}

sub _get_time {
    my ($seconds, $minutes, $hours,   $day, $month,
        $year,    $weekday, $yearday, $isSummertime
    ) = localtime(time);

    $year  += 1900;
    $month += 1;

    return sprintf "%d-%02d-%02d %02d:%02d:%02d", $year, $month, $day, $hours,
        $minutes, $seconds;
}

sub _backup_logfile {
    my ($self) = @_;
    my $filesize = $self->_get_logfile_size();

    if ( $filesize >= $self->{logfile_max_size} ) {
        my $backup_filename = $self->_get_logfile_backup_name();
        copy( $self->{filename}, $backup_filename );
        unlink $self->{filename};
        return 1;
    }

    return 0;
}

sub _get_logfile_size {
    my ($self) = @_;

    return ( stat $self->{filename} )[7];
}

sub _get_logfile_backup_name {
    my ($self) = @_;

    return $self->{filename} . '.bak';
}

sub _print_log_logfile_backuped {
    my ($self) = @_;
    my $backup_filename = $self->_get_logfile_backup_name();

    $self->log_print(
        "Die Logdatei hat die Maximalgroesse von $self->{logfile_max_size}"
            . " Bytes ueberschritten\n"
            . "und wurde kopiert nach '$backup_filename'.\n"
            . "Sollten Sie Wert auf die Informationen darin legen, sichern Sie"
            . " bitte diese Backupdatei!" );
}

1;

__END__

=head1 NAME

Util::Logfile - Logdatei, die auch auf die Standardausgabe ausgibt

=head1 DESCRIPTION

Util::Logfile ist eine Logdatei, die auch auf die Standardausgabe ausgibt.
Sie kann anstelle von B<print> benutzt werden, das Schreiben in eine Logdatei
ist optional. Meldungen mssen I<nicht> mit einem Newline abgeschlossen werden,
das erledigen die Logging-Funktionen.

Sie ist ein Ersatz fr L<http://cpan.uwinnipeg.ca/htdocs/Log-Logger/Log/Logger.html>
(Klasse B<Log::Logger>) und kann anstelle dieser benutzt werden.

B<Log::Logger> versucht auch nicht geffnete Filehandles zu
benutzen (Flush), schreibt keine Uhrzeit in die Logdatei und man kann nicht
definieren, ob der Output ein Fehler ist (Ausgabe auf STDERR).

=head1 METHODS

=over

=item $log = Util::Logfile->new($filename, $append)

Erzeugt ein neues Objekt.

Der 1. Parameter ist optional und ein String mit dem Dateinamen. Ist er
definiert, wird die Datei geffnet.

Der 2. Parameter ist ein Boolean und true, falls die Ausgabe angehngt
werden soll. Ist er false, wird eine existierende Logdatei gleichen Namens
berschrieben. Fehlt der Parameter, wird eine existierende Datei ebenfalls
berschrieben.

=item $success = $log->open($filename, $append)

ffnet eine Datei zum Loggen. Eine bereits geffnete wird vorher geschlossen.

Der 1. Parameter ist ein String mit dem Dateinamen

Der 2. Parameter ist ein Boolean und true, falls die Ausgabe angehngt
werden soll. Ist er false, wird eine existierende Logdatei gleichen Namens
berschrieben. Fehlt der Parameter, wird eine existierende Datei ebenfalls
berschrieben.

Der Rckgabewert ist true bei Erfolg, sonst false

=item $success $log->open_append($filename)

Beibehalten aus Kompatiblittsgrnden. Ruft $log->open_append($filename, 1) auf.

=item $log->log($message, $error)

Schreibt eine Nachricht (nur) in die Logdatei, sofern zum Schreiben geffnet.

Der 1. Parameter ist ein String mit der Nachricht, in der Logdatei wird
auerdem die Systemzeit vorangestellt im Format (YYYY-MM-DD HH:MM:SS)

Der 2. Parameter ist ein Boolean und true im Fehlerfall. In die Logdatei
wird in diesem Fall der Nachricht zustzlich der Text C<[Error]> vorangestellt.

=item $log->log_print($message, $error)

Schreibt die Nachricht in die Logdatei und gibt sie aus, im Fehlerfall
auf STDERR, sonst auf STDOUT.

Die Parameter sind beschrieben unter B<log()>. 

=item $log->fail($message, $exit_value)

Loggt eine Nachricht und beendet die Programmausfhrung via B<exit()>.

Der 1. Parameter ist ein String mit der Nachricht

Der 2. Parameter ist der Wert, mit dem das Programm beendet wird;
wird dieser ausgelassen, ist der Exit-Wert 1

=item $log->set_logfile_max_size($max_size)

Setzt die maximale Gre der Logdatei. Ist diese erreicht,
wird eine Backupdatei gleichen Namens mit der Endung '.bak'
erzeugt und danach die Logdatei gelscht.

Der 1. Parameter ist die maximale Gre in Bytes. Default: 1 MB.

=item $log->set_verbose($verbose)

Setzt, ob mit C<log_verbose()> Meldungen ausgegeben werden sollen.
Default: false.

Der erste Parameter ist ein Boolean

=item $log->log_verbose($message)

Gibt eine Nachricht auf die Standardausgabe aus, sofern der Verobse-Modus
gesetzt wurde mit C<set_verbose()>.

Der 1. Parameter ist ein String mit der Nachricht

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 

=cut
