#!/usr/bin/perl -w

use strict;
use Readonly;
use Text::Iconv;
use Test::More qw( no_plan );

use Converter::XmpDataToIptcData;
use Util::Object;

require 'globals.pl';

Readonly my %XMP_HASH => (
    AllXmpData => {
        'exif:DateTimeOriginal' => '2007-10-11T18:39:48.70+02:00',
        'text_encoding'         => 'ISO-8859-1',
        'dc:creator'            => ['creator '],
        'dc:description'        => ['description '],
        'dc:rights'             => ['rights '],
        'dc:subject'            => [
            'Haus ',
            'Dach ',
            'Schornstein '
        ],
        'dc:title'                 => ['title '],
        'Iptc4xmpCore:Location'    => 'location ',
        'Iptc4xmpCore:CountryCode' => 'cco',
        'photoshop:Headline'       => 'headline',
        'photoshop:CaptionWriter'  => 'caption writer',
        'photoshop:Category'       => 'cat',
        'photoshop:SupplementalCategories' =>
            [ 'supplemental category 1', 'supplemental category 2' ],
        'photoshop:City'                  => 'city',
        'photoshop:State'                 => 'state',
        'photoshop:Country'               => 'country',
        'photoshop:TransmissionReference' => 'transmisssion reference',
        'photoshop:Instructions'          => 'instructions',
        'photoshop:Credit'                => 'credit',
        'photoshop:Source'                => 'source',
        'photoshop:AuthorsPosition'       => 'position',
    },
    OnlyTitle => {
        'dc:title'      => ['title '],
        'text_encoding' => 'ISO-8859-1'
    },
);

test_empty_xmp_hash();
test_text_encodings();
test_multiple_iptc_values_for_one_xmp_key();

sub get_converted_array {
    my ( $array_ref, $text_encoding ) = @_;
    my @konverted_array = ();

    foreach my $element ( @{$array_ref} ) {
        push @konverted_array, get_converted( $element, $text_encoding );
    }

    return @konverted_array;
}

sub get_converted {
    my ( $string, $text_encoding ) = @_;
    my $iconv = Text::Iconv->new( 'ISO-8859-1', $text_encoding );

    return $iconv->convert($string);
}

sub test {
    my ( $xmp_hash_ref, $text_encoding ) = @_;
    my $converter     = Converter::XmpDataToIptcData->new($text_encoding);
    my $iptc_data_ref = $converter->get_iptc_data_from_xmp_data($xmp_hash_ref);

    foreach my $xmp_key ( keys %main::IPTC_KEY_FOR_XMP_KEY ) {
        next if has_xmp_key_multiple_iptc_keys($xmp_key);
        my $iptc_key  = $main::IPTC_KEY_FOR_XMP_KEY{$xmp_key};
        my $test_name = "[$xmp_key] -> [$iptc_key] ($text_encoding)";
        my @xmp_array = ();
        my $xmp_value;
        if ( not defined $xmp_hash_ref->{$xmp_key} ) {
            is( $iptc_data_ref->{$iptc_key}, undef, $test_name );
            next;
        }
        if ( main::is_xmp_value_repeatable($xmp_key) ) {
            @xmp_array = get_converted_array( $xmp_hash_ref->{$xmp_key},
                $text_encoding );
            $xmp_value = $xmp_array[0];
        }
        else {
            $xmp_value
                = get_converted( $xmp_hash_ref->{$xmp_key}, $text_encoding );
            push @xmp_array, $xmp_value;
        }
        if ( main::is_iptc_value_repeatable($iptc_key) ) {
            ok( Util::Array::elements_equals(
                    $iptc_data_ref->{$iptc_key},
                    \@xmp_array
                ),
                $test_name
            );
        }
        else {
            my $iptc_value = $iptc_data_ref->{$iptc_key};
            is( $iptc_value, $xmp_value, $test_name );
        }
    }
}

sub test_multiple_iptc_values_for_one_xmp_key {
    test_date_time( $XMP_HASH{AllXmpData} );
}

sub test_date_time {
    my ($xmp_hash_ref) = @_;
    my $converter      = Converter::XmpDataToIptcData->new();
    my $iptc_data_ref  = $converter->get_iptc_data_from_xmp_data($xmp_hash_ref);

    is( $iptc_data_ref->{'2:62 Digital Creation Date'}, '20071011', 'Datum' );
    is( $iptc_data_ref->{'2:63 Digital Creation Time'},
        '183948+0200', 'Uhrzeit' );
}

sub test_empty_xmp_hash {
    my %empty_xmp_data = ();
    my $converter      = Converter::XmpDataToIptcData->new('ISO-8859-1');
    my @iptc_keys      = keys %main::IPTC_INFO;

    $empty_xmp_data{text_encoding} = 'ISO-8859-1';
    my $iptc_data = $converter->get_iptc_data_from_xmp_data( \%empty_xmp_data );

    foreach my $iptc_key (@iptc_keys) {
        my $test_name = "[$iptc_key] Empty XMP-Hash";
        is( $iptc_data->{$iptc_key}, undef, $test_name );
    }
}

sub test_text_encodings {
    foreach my $key ( keys %XMP_HASH ) {
        test( $XMP_HASH{$key}, 'ISO-8859-1' );
        test( $XMP_HASH{$key}, 'UTF-8' );
    }
}

