#!/usr/bin/perl

use version; our $VERSION = qv('0.5_5');
use CONFIG::Hash;
use File::Basename qw(dirname);
use lib dirname(__FILE__);
use Util::Logfile;
use Getopt::Std;
use strict;

use Converter::XmpDataToIptcData;
use Image::File;
use IPTC::Factory;
use Language::Messages;
use Util::Object;
use Util::File;
use XMP::ImageFileFinder;
use XMP::Reader;

my $config_filename;
my %commandline_option = ();
my $message_file       = CONFIG::Hash->new( get_messagefilename() );
my $messages           = Language::Messages->new( $message_file->get() );

getopts( 'c:v', \%commandline_option );
read_configfile();
init_logger();

my $update_count = 0;
my $iptc_writer  = IPTC::Factory::create_writer();
my $reader       = IPTC::Factory::create_reader();

log_config_file();
update_iptc();
log_update_count();

sub update_iptc {
    my $finder = XMP::ImageFileFinder->new();

    $finder->find_imagefiles( \@main::XMP_PATHS, \@main::IMG_SUFFIXES );
    my @imagefilenames = $finder->get_imagefilenames();

    foreach my $imagefilename (@imagefilenames) {
        my $xmp_filename = Image::File::get_xmp_file_name($imagefilename);
        if ( Util::File::is_newer( $xmp_filename, $imagefilename ) ) {
            $main::log->log_verbose(
                "updateiptc: Aktualisiere Bild '$imagefilename'");
            set_iptc_header( $xmp_filename, $imagefilename );
        }
    }
}

sub set_iptc_header {
    my ( $xmp_filename, $imagefilename ) = @_;
    my $converter  = Converter::XmpDataToIptcData->new('ISO-8859-1');
    my $xmp_reader = XMP::Reader->new();

    $xmp_reader->read_file($xmp_filename);
    my $iptc_data = $converter->get_iptc_data_from_xmp_data(
        $xmp_reader->get_xmp_data() );

    if ( not defined $iptc_data ) {
        $main::log->log_print( 'update_xmp_empty', $imagefilename );
        return;
    }

    if (!$main::WRITE_ALWAYS_IF_XMPFILE_IS_NEWER
        && iptc_info_equals_xmp_info( $xmp_filename, $imagefilename,
            $iptc_data )
        )
    {
        $main::log->log_print(
            $messages->get_message(
                'update_xmp_equals_iptc', $xmp_filename, $imagefilename
            )
        );
        Image::File::set_equal_time_to_xmp_file($imagefilename);
    }
    elsif ( $iptc_writer->write_file( $imagefilename, $iptc_data ) ) {
        $main::log->log_print(
            $messages->get_message( 'update_success', $imagefilename ) );
        $update_count++;
    }
    else {
        $main::log->log_print(
            $messages->get_message(
                'update_err_write', $xmp_filename, $imagefilename
            )
        );
    }
}

sub iptc_info_equals_xmp_info {
    my ( $xmp_filename, $imagefilename, $iptc_data ) = @_;

    $reader->read_file($imagefilename);
    my $read_iptc_data = $reader->get_iptc_data();

    $read_iptc_data->{text_encoding} = $iptc_data->{text_encoding};

    return Util::Object::equals( $iptc_data, $read_iptc_data );
}

sub get_config_filename {
    if ( defined $commandline_option{c} ) {
        my $filename = $commandline_option{c};
        die "Konfigurationsatei '$filename' existiert nicht!"
            if not -f $filename;
        return $filename;
    }
    return 'config.pl';
}

sub get_messagefilename {
    die "Messagefile 'res/messages_de_DE' if not found!"
        if not -f dirname(__FILE__) . '/res/messages_de_DE';

    return dirname(__FILE__) . '/res/messages_de_DE';
}

sub init_logger {
    create_logger();
    if ( -d $main::LOGFILE_DIR ) {
        if ( -w $main::LOGFILE_DIR ) {
            my $logfilename = "$main::LOGFILE_DIR/updateiptc-log.txt";
            $messages->get_message( 'udate_err_logfile_not_opened',
                $logfilename, $config_filename )
                if !$main::log->open_append($logfilename);
            print $messages->get_message( 'update_logfile', $logfilename )
                . "\n";
        }
        else {
            $messages->get_message( 'udate_err_logfile_dir_not_writable',
                $main::LOGFILE_DIR, $config_filename );
        }
    }
    else {
        $messages->get_message( 'udate_err_logfile_dir_not_exists',
            $main::LOGFILE_DIR, $config_filename );
    }
}

sub create_logger {
    if ( not defined $main::log ) {
        $main::log = Util::Logfile->new();
    }

    my $verbose = ( defined $commandline_option{v} ? 1 : 0 );
    $main::log->set_verbose($verbose);
    $main::log->log_verbose("updateiptc: Zeige ausfhrliche Meldungen an");
}

sub read_configfile {
    $config_filename = get_config_filename();
    require $config_filename;
}

sub log_config_file {
    $main::log->log_print(
        $messages->get_message( 'update_configfile', $config_filename ) );
}

sub log_update_count {
    $main::log->log_print(
        $messages->get_message( 'update_count', $update_count ) );
}

1;

__END__

=head1 NAME

updateiptc - Aktualisiert die IPTC-Informationen eines Bilds anhand einer
XMP-Datei

=head1 DESCRIPTION

Das Skript C<updateiptc> sucht in allen Verzeichnissen, die in
C<@main::XMP_PATHS> definiert sind, nach C<XMP>-Dateien. Das sind Dateien mit
der Endung C<xmp>. Findet es im gleichen Verzeichnis eine Bilddatei mit
gleichem Basisnamen, (z.B. C<bild-12.xmp> und C<bild-12.jpg>), wird geprft,
ob die C<XMP>-Datei neuer ist als die Bilddatei. Als Bilddatei wird jede Datei
betrachtet mit einer Endung, die in C<@main::IMG_SUFFIXES> steht.

Ist die XMP-Datei neuer als die Bilddatei, werden die IPTC-Informationen der
Bilddatei modifiziert anhand der XMP-Datei. I<Neuer> gilt in Bezug auf die Zeit
der letzten Modifikation.  

Die globalen Variablen werden gelesen aus der Konfigurationsdatei C<config.pl>.

=head1 OPTIONS

Es gibt folgende Kommandozeilenoptionen:

=over

=item -c Konfigurationsdatei

Alternative Konfigurationsdatei anstelle von C<config.pl>

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/05/17

=cut
